@@ -46,10 +46,16 @@ function(x, Criterion, pos, ...)
   theta2 <- unlist(x@Theta[[pos]][grep("theta2", Names)])
 
   theta2[is.na(theta2)] <- 0
+  
+  theta3 <- unlist(x@Theta[[pos]][grep("theta3", Names)])
+
+  theta3[is.na(theta3)] <- 0  
 
   length(pdf) <- d
 
   C <- x@summary[pos, "Preprocessing"]
+  
+  if (is.na(C)) C <- "NA" 
 
   if (C == .rebmix$Preprocessing[1]) {
     y0 <- as.double(x@summary[pos, paste("y0", if (d > 1) 1:d else "", sep = "")])
@@ -66,10 +72,10 @@ function(x, Criterion, pos, ...)
       c = as.integer(c),
       w = as.double(x@w[[pos]]),
       length.pdf = as.integer(d),
-      length.Theta = as.integer(2),
-      length.theta = as.integer(c(d, d)),
+      length.Theta = as.integer(3),
+      length.theta = as.integer(c(d, d, d)),
       pdf = as.character(pdf),
-      Theta = as.double(c(theta1, theta2)),
+      Theta = as.double(c(theta1, theta2, theta3)),
       n = as.integer(n),
       x = as.double(X),
       IC = double(1),
@@ -91,10 +97,10 @@ function(x, Criterion, pos, ...)
       c = as.integer(c),
       w = as.double(x@w[[pos]]),
       length.pdf = as.integer(d),
-      length.Theta = as.integer(2),
-      length.theta = as.integer(c(d, d)),
+      length.Theta = as.integer(3),
+      length.theta = as.integer(c(d, d, d)),
       pdf = as.character(pdf),
-      Theta = as.double(c(theta1, theta2)),
+      Theta = as.double(c(theta1, theta2, theta3)),
       n = as.integer(n),
       x = as.double(X),
       IC = double(1),
@@ -119,10 +125,10 @@ function(x, Criterion, pos, ...)
       c = as.integer(c),
       w = as.double(x@w[[pos]]),
       length.pdf = as.integer(d),
-      length.Theta = as.integer(2),
-      length.theta = as.integer(c(d, d)),
+      length.Theta = as.integer(3),
+      length.theta = as.integer(c(d, d, d)),
       pdf = as.character(pdf),
-      Theta = as.double(c(theta1, theta2)),
+      Theta = as.double(c(theta1, theta2, theta3)),
       n = as.integer(n),
       x = as.double(X),
       IC = double(1),
@@ -194,6 +200,8 @@ function(x, Criterion, pos, ...)
   length(pdf) <- d
 
   C <- x@summary[pos, "Preprocessing"]
+  
+  if (is.na(C)) C <- "NA" 
 
   if (C == .rebmix$Preprocessing[1]) {
     y0 <- as.double(x@summary[pos, paste("y0", if (d > 1) 1:d else "", sep = "")])
