@@ -1,4 +1,4 @@
-.densHistogram.x <- function(k, x, x0, hx, cx, px)
+.densHistogram.x <- function(k, x, x0, xmin, xmax, hx, cx, px)
 {
   output <- .C(C_RdensHistogramX,
     k = as.integer(k),
@@ -6,6 +6,8 @@
     x = as.double(x),
     y = double(length(x)),
     x0 = as.double(x0),
+    xmin = as.double(xmin),    
+    xmax = as.double(xmax),
     hx = as.double(hx),
     px = as.character(px),
     error = integer(1),
