@@ -73,6 +73,8 @@ function(x,
   theta1 <- Theta[grep("theta1", Names)]
 
   theta2 <- Theta[grep("theta2", Names)]
+  
+  theta3 <- Theta[grep("theta3", Names)]
 
   f <- array(data = 0.0, dim = n, dimnames = NULL)
 
@@ -108,6 +110,10 @@ function(x,
         fi <- fi * dgamma(as.numeric(Dataset[, j]), scale = as.numeric(theta1[[i]][j]), shape = as.numeric(theta2[[i]][j]), ...)
       }
       else
+      if (pdf[[i]][j] == .rebmix$pdf[8]) {
+        fi <- fi * dunif(as.numeric(Dataset[, j]), min = as.numeric(theta1[[i]][j]), max = as.numeric(theta2[[i]][j]), ...)
+      }
+      else      
       if (pdf[[i]][j] == .rebmix$pdf[9]) {
         output <- .C(C_RvonMisesPdf,
           n = as.integer(n),
@@ -125,7 +131,8 @@ function(x,
           n = as.integer(n),
           y = as.double(Dataset[, j]),
           Mean = as.double(theta1[[i]][j]),
-          Beta = as.double(theta2[[i]][j]),
+          Sigma = as.double(theta2[[i]][j]),
+          Xi = as.double(theta3[[i]][j]),
           f = double(n),
           PACKAGE = "rebmix")
 
