@@ -1,5 +1,4 @@
 ### Panic Branislav.
-
 setMethod("a.strategy", signature(x = "EM.Control"), function(x) x@strategy)
 setMethod("a.variant", signature(x = "EM.Control"), function(x) x@variant)
 setMethod("a.acceleration", signature(x = "EM.Control"), function(x) x@acceleration)
@@ -7,6 +6,7 @@ setMethod("a.tolerance", signature(x = "EM.Control"), function(x) x@tolerance)
 setMethod("a.acceleration.multiplier", signature(x = "EM.Control"), function(x) x@acceleration.multiplier)
 setMethod("a.maximum.iterations", signature(x = "EM.Control"), function(x) x@maximum.iterations)
 setMethod("a.K", signature(x = "EM.Control"), function(x) x@K)
+setMethod("a.eliminate.zero.components", signature(x = "EM.Control"), function(x) x@eliminate.zero.components)
 
 setMethod("a.strategy<-",
           signature = (x = "EM.Control"),
@@ -179,6 +179,28 @@ function(x, value)
   x
 }) ## a.K<-
 
+setMethod("a.eliminate.zero.components<-",
+          signature = (x = "EM.Control"),
+function(x, value)
+{
+  # value.
+  
+  if (missing(value) || (length(value) == 0)) {
+    stop(sQuote("value"), " must not be empty!", call. = FALSE)
+  }
+
+  if (!is.logical(value)) {
+    stop(sQuote("value"), " logical is requested!", call. = FALSE)
+  }
+
+  length(value) <- 1
+
+  x@eliminate.zero.components <- value
+
+  rm(list = ls()[!(ls() %in% c("x"))])
+
+  x
+}) ## a.eliminate.zero.components<-
 ### End
 
 setMethod("a.c", signature(x = "RNGMIX.Theta"), function(x) x@c)
@@ -186,6 +208,70 @@ setMethod("a.d", signature(x = "RNGMIX.Theta"), function(x) x@d)
 setMethod("a.pdf", signature(x = "RNGMIX.Theta"), function(x) x@pdf)
 setMethod("a.Theta", signature(x = "RNGMIX.Theta"), function(x) x@Theta)
 
+### Panic Branislav.
+setMethod("a.w", signature(x = "EMMIX.Theta"), function(x) x@w)
+
+setMethod("a.w<-", 
+          signature(x = "EMMIX.Theta"), 
+function(x, value)
+{
+  if (missing(value) || (length(value) == 0)) {
+    stop(sQuote("value"), " must not be empty!", call. = FALSE)
+  }
+
+  if (!is.number(value)) {
+    stop(sQuote("value"), " numeric vector is requested!", call. = FALSE)
+  }
+
+  if (length(value) != x@c) {
+    stop("length of ", sQuote("value"), " must equal " , x@c, "!", call. = FALSE)
+  }
+
+  if (abs(sum(value) - 1.0) > 1e-6){
+    stop(sQuote("value"), " must sum to 1.0!", call. = FALSE) 
+  }
+
+  value <- value/sum(value)
+
+  x@w <- value
+
+  rm(list = ls()[!(ls() %in% c("x"))])
+
+  x
+}) ## a.w<-
+
+setMethod("a.w", signature(x = "EMMVNORM.Theta"), function(x) x@w)
+
+setMethod("a.w<-", 
+          signature(x = "EMMVNORM.Theta"), 
+function(x, value)
+{
+  if (missing(value) || (length(value) == 0)) {
+    stop(sQuote("value"), " must not be empty!", call. = FALSE)
+  }
+
+  if (!is.number(value)) {
+    stop(sQuote("value"), " numeric vector is requested!", call. = FALSE)
+  }
+
+  if (length(value) != x@c) {
+    stop("length of ", sQuote("value"), " must equal " , x@c, "!", call. = FALSE)
+  }
+
+  if (abs(sum(value) - 1.0) > 1e-6){
+    stop(sQuote("value"), " must sum to 1.0!", call. = FALSE) 
+  }
+
+  value <- value/sum(value)
+
+  x@w <- value
+
+  rm(list = ls()[!(ls() %in% c("x"))])
+
+  x
+}) ## a.w<-
+### End
+
 setMethod("a.theta1<-",
           signature(x = "RNGMIX.Theta", l = "missing"),
 function(x, value)
@@ -208,7 +294,7 @@ function(x, value)
 
   for (l in 1:x@c) {
     if (x@pdf == .rebmix$pdf[3]) {
-      if (value[l] < 0.0) {
+      if (value[l] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[3]), " must be greater than 0.0!", call. = FALSE)
       }
     }
@@ -224,22 +310,29 @@ function(x, value)
     }
     else
     if (x@pdf == .rebmix$pdf[5]) {
-      if (value[l] < 0.0) {
+      if (value[l] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[5]), " must be greater than 0.0!", call. = FALSE)
       }
     }
     else
     if (x@pdf == .rebmix$pdf[7]) {
-      if (value[l] < 0.0) {
+      if (value[l] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[7]), " must be greater than 0.0!", call. = FALSE)
       }
     }
+    else
+    if (x@pdf == .rebmix$pdf[8]) {
+      if (value[l] >= x@Theta[[3 + (l - 1) * 4]]) {
+        stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[8]), " must be less than ", x@Theta[[3 + (l - 1) * 4]], "!", call. = FALSE)        
+      }
+    }    
   }
 
   for (l in 1:x@c) {
-    length(x@Theta[[1 + (l - 1) * 3]]) <- 1
-    x@Theta[[2 + (l - 1) * 3]] <- value[l]
-    length(x@Theta[[3 + (l - 1) * 3]]) <- 1
+    length(x@Theta[[1 + (l - 1) * 4]]) <- 1
+    x@Theta[[2 + (l - 1) * 4]] <- value[l]
+    length(x@Theta[[3 + (l - 1) * 4]]) <- 1
+    length(x@Theta[[4 + (l - 1) * 4]]) <- 1
   }
 
   rm(list = ls()[!(ls() %in% c("x"))])
@@ -283,7 +376,7 @@ function(x, l, value)
 
   for (i in 1:x@d) {
     if (x@pdf[i] == .rebmix$pdf[3]) {
-      if (value[i] < 0.0) {
+      if (value[i] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[3]), " must be greater than 0.0!", call. = FALSE)
       }
     }
@@ -299,19 +392,25 @@ function(x, l, value)
     }
     else
     if (x@pdf[i] == .rebmix$pdf[5]) {
-      if (value[i] < 0.0) {
+      if (value[i] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[5]), " must be greater than 0.0!", call. = FALSE)
       }
     }
     else
     if (x@pdf[i] == .rebmix$pdf[7]) {
-      if (value[i] < 0.0) {
+      if (value[i] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[7]), " must be greater than 0.0!", call. = FALSE)
       }
     }
+    else
+    if (x@pdf[i] == .rebmix$pdf[8]) {
+      if (value[i] >= x@Theta[[3 + (l - 1) * 4]][i]) {
+        stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[8]), " must be less than ", x@Theta[[3 + (l - 1) * 4]][i], "!", call. = FALSE)        
+      }
+    }    
   }
 
-  x@Theta[[2 + (l - 1) * 3]] <- value
+  x@Theta[[2 + (l - 1) * 4]] <- value
 
   rm(list = ls()[!(ls() %in% c("x"))])
 
@@ -340,19 +439,19 @@ function(x, value)
 
   for (l in 1:x@c) {
     if (x@pdf == .rebmix$pdf[1]) {
-      if (value[l] < 0.0) {
+      if (value[l] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[1]), " must be greater than 0.0!", call. = FALSE)
       }
     }
     else
     if (x@pdf == .rebmix$pdf[2]) {
-      if (value[l] < 0.0) {
+      if (value[l] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[2]), " must be greater than 0.0!", call. = FALSE)
       }
     }
     else
     if (x@pdf == .rebmix$pdf[3]) {
-      if (value[l] < 0.0) {
+      if (value[l] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[3]), " must be greater than 0.0!", call. = FALSE)
       }
     }
@@ -372,28 +471,35 @@ function(x, value)
     }
     else
     if (x@pdf == .rebmix$pdf[7]) {
-      if (value[l] < 0.0) {
+      if (value[l] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[7]), " must be greater than 0.0!", call. = FALSE)
       }
     }
     else
+    if (x@pdf == .rebmix$pdf[8]) {
+      if (value[l] <= x@Theta[[2 + (l - 1) * 4]]) {
+        stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[8]), " must be greater than ", x@Theta[[2 + (l - 1) * 4]], "!", call. = FALSE)        
+      }
+    }
+    else
     if (x@pdf == .rebmix$pdf[9]) {
-      if (value[l] < 0.0) {
+      if (value[l] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[9]), " must be greater than 0.0!", call. = FALSE)
       }
     }
     else
     if (x@pdf == .rebmix$pdf[10]) {
-      if (value[l] < 0.0) {
+      if (value[l] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[10]), " must be greater than 0.0!", call. = FALSE)
       }
     }    
   }
 
   for (l in 1:x@c) {
-    length(x@Theta[[1 + (l - 1) * 3]]) <- 1
-    length(x@Theta[[2 + (l - 1) * 3]]) <- 1
-    x@Theta[[3 + (l - 1) * 3]] <- value[l]
+    length(x@Theta[[1 + (l - 1) * 4]]) <- 1
+    length(x@Theta[[2 + (l - 1) * 4]]) <- 1
+    x@Theta[[3 + (l - 1) * 4]] <- value[l]
+    length(x@Theta[[4 + (l - 1) * 4]]) <- 1
   }
 
   rm(list = ls()[!(ls() %in% c("x"))])
@@ -401,6 +507,49 @@ function(x, value)
   x
 }) ## a.theta2<-
 
+setMethod("a.theta3<-",
+          signature(x = "RNGMIX.Theta", l = "missing"),
+function(x, value)
+{
+  x@d <- 1; length(x@pdf) <- 1
+
+  # value.
+
+  if (missing(value) || (length(value) == 0)) {
+    stop(sQuote("value"), " must not be empty!", call. = FALSE)
+  }
+
+  if (!is.numeric(value)) {
+    stop(sQuote("value"), " numeric vector is requested!", call. = FALSE)
+  }
+
+  if (length(value) != x@c) {
+    stop("length of ", sQuote("value"), " must equal " , x@c, "!", call. = FALSE)
+  }
+
+  for (l in 1:x@c) {
+    if (x@pdf == .rebmix$pdf[10]) {
+      if (abs(abs(value[l]) - 1.0) > .Machine$double.eps^0.5) {
+        stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[10]), " must be -1.0 or 1.0!", call. = FALSE)
+      }
+    }
+    else {
+      value[l] <- NA
+    }    
+  }
+
+  for (l in 1:x@c) {
+    length(x@Theta[[1 + (l - 1) * 4]]) <- 1
+    length(x@Theta[[2 + (l - 1) * 4]]) <- 1
+    length(x@Theta[[3 + (l - 1) * 4]]) <- 1
+    x@Theta[[4 + (l - 1) * 4]] <- value[l]
+  }
+
+  rm(list = ls()[!(ls() %in% c("x"))])
+
+  x
+}) ## a.theta3<-
+
 setMethod("a.theta2<-",
           signature(x = "RNGMIX.Theta"),
 function(x, l, value)
@@ -437,19 +586,19 @@ function(x, l, value)
 
   for (i in 1:x@d) {
     if (x@pdf[i] == .rebmix$pdf[1]) {
-      if (value[i] < 0.0) {
+      if (value[i] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[1]), " must be greater than 0.0!", call. = FALSE)
       }
     }
     else
     if (x@pdf[i] == .rebmix$pdf[2]) {
-      if (value[i] < 0.0) {
+      if (value[i] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[2]), " must be greater than 0.0!", call. = FALSE)
       }
     }
     else
     if (x@pdf[i] == .rebmix$pdf[3]) {
-      if (value[i] < 0.0) {
+      if (value[i] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[3]), " must be greater than 0.0!", call. = FALSE)
       }
     }
@@ -469,31 +618,89 @@ function(x, l, value)
     }
     else
     if (x@pdf[i] == .rebmix$pdf[7]) {
-      if (value[i] < 0.0) {
+      if (value[i] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[7]), " must be greater than 0.0!", call. = FALSE)
       }
     }
     else
+    if (x@pdf[i] == .rebmix$pdf[8]) {
+      if (value[i] <= x@Theta[[2 + (l - 1) * 4]][i]) {
+        stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[8]), " must be greater than ", x@Theta[[2 + (l - 1) * 4]][i], "!", call. = FALSE)
+      }
+    }    
+    else
     if (x@pdf[i] == .rebmix$pdf[9]) {
-      if (value[i] < 0.0) {
+      if (value[i] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[9]), " must be greater than 0.0!", call. = FALSE)
       }
     }
     else
     if (x@pdf[i] == .rebmix$pdf[10]) {
-      if (value[i] < 0.0) {
+      if (value[i] <= 0.0) {
         stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[10]), " must be greater than 0.0!", call. = FALSE)
       }
     }    
   }
 
-  x@Theta[[3 + (l - 1) * 3]] <- value
+  x@Theta[[3 + (l - 1) * 4]] <- value
 
   rm(list = ls()[!(ls() %in% c("x"))])
 
   x
 }) ## a.theta2<-
 
+setMethod("a.theta3<-",
+          signature(x = "RNGMIX.Theta"),
+function(x, l, value)
+{
+  # l.
+
+  if (missing(l) || (length(l) == 0)) {
+    stop(sQuote("l"), " must not be empty!", call. = FALSE)
+  }
+
+  if (!is.wholenumber(l)) {
+    stop(sQuote("l"), " integer is requested!", call. = FALSE)
+  }
+
+  length(l) <- 1
+
+  if ((l < 1) || (l > x@c)) {
+    stop(sQuote("l"), " must be greater than 0 and less or equal than ", x@c, "!", call. = FALSE)
+  }
+
+  # value.
+
+  if (missing(value) || (length(value) == 0)) {
+    stop(sQuote("value"), " must not be empty!", call. = FALSE)
+  }
+
+  if (!is.numeric(value)) {
+    stop(sQuote("value"), " numeric vector is requested!", call. = FALSE)
+  }
+
+  if (length(value) != x@d) {
+    stop("length of ", sQuote("value"), " must equal " , x@d, "!", call. = FALSE)
+  }
+
+  for (i in 1:x@d) {
+    if (x@pdf[i] == .rebmix$pdf[10]) {
+      if (abs(abs(value[i]) - 1.0) > .Machine$double.eps^0.5) {
+        stop(sQuote("value"), " for ", dQuote(.rebmix$pdf[10]), " must be -1.0 or 1.0!", call. = FALSE)
+      }    
+    }
+    else {
+      value[i] <- NA 
+    }    
+  }
+
+  x@Theta[[4 + (l - 1) * 4]] <- value
+
+  rm(list = ls()[!(ls() %in% c("x"))])
+
+  x
+}) ## a.theta3<-
+
 setMethod("a.theta1<-",
           signature(x = "RNGMVNORM.Theta"),
 function(x, l, value)
@@ -576,6 +783,93 @@ function(x, l, value)
   x
 }) ## a.theta2<-
 
+### Panic Branislav.
+setMethod("a.theta1.all<-", 
+          signature(x = "RNGMIX.Theta"),
+function(x, value)
+{
+  i = 1
+
+  for (j in 1:x@c) {
+    a.theta1(x, j) <- value[i:(i + x@d - 1)]
+    
+    i = i + x@d
+  }
+
+  rm(list = ls()[!(ls() %in% c("x"))])
+
+  x
+})## a.theta1.all
+
+setMethod("a.theta2.all<-", 
+          signature(x = "RNGMIX.Theta"),
+function(x, value)
+{
+  i = 1
+
+  for (j in 1:x@c) {
+    a.theta2(x, j) <- value[i:(i + x@d - 1)]
+    
+    i = i + x@d
+  }
+
+  rm(list = ls()[!(ls() %in% c("x"))])
+
+  x
+})
+
+setMethod("a.theta3.all<-", 
+          signature(x = "RNGMIX.Theta"),
+function(x, value)
+{
+  i = 1
+
+  for (j in 1:x@c) {
+    a.theta3(x, j) <- value[i:(i + x@d - 1)]
+    
+    i = i + x@d
+  }
+
+  rm(list = ls()[!(ls() %in% c("x"))])
+
+  x
+}) ## a.theta3.all<-
+
+setMethod("a.theta1.all<-", 
+          signature(x = "RNGMVNORM.Theta"),
+function(x, value)
+{
+  i = 1
+
+  for (j in 1:x@c) {
+    a.theta1(x, j) <- value[i:(i + x@d - 1)]
+    
+    i = i + x@d
+  }
+
+  rm(list = ls()[!(ls() %in% c("x"))])
+
+  x
+}) ## a.theta1.all<-
+
+setMethod("a.theta2.all<-", 
+          signature(x = "RNGMVNORM.Theta"),
+function(x, value)
+{
+  i = 1
+
+  for (j in 1:x@c) {
+    a.theta2(x, j) <- value[i:(i + x@d * x@d - 1)]
+    
+    i = i + x@d * x@d
+  }
+
+  rm(list = ls()[!(ls() %in% c("x"))])
+
+  x
+}) ## a.theta2.all<-
+### End
+
 setMethod("a.Dataset.name", signature(x = "RNGMIX"), function(x) x@Dataset.name)
 setMethod("a.rseed", signature(x = "RNGMIX"), function(x) x@rseed)
 setMethod("a.n", signature(x = "RNGMIX"), function(x) x@n)
@@ -639,6 +933,7 @@ setMethod("a.Variables", signature(x = "REBMIX"), function(x) x@Variables)
 setMethod("a.pdf", signature(x = "REBMIX"), function(x) x@pdf)
 setMethod("a.theta1", signature(x = "REBMIX"), function(x) x@theta1)
 setMethod("a.theta2", signature(x = "REBMIX"), function(x) x@theta2)
+setMethod("a.theta3", signature(x = "REBMIX"), function(x) x@theta2)
 
 setMethod("a.theta1.all",
           signature(x = "REBMIX"),
@@ -659,11 +954,11 @@ function(x, pos)
   output <- matrix(data = 0.0, nrow = c, ncol = d, byrow = TRUE)
 
   for (l in 1:c) {
-    output[l, ] <- x@Theta[[pos]][[2 + (l - 1) * 3]]
+    output[l, ] <- x@Theta[[pos]][[2 + (l - 1) * 4]]
   }
 
   colnames(output) <- NULL
-  rownames(output) <- paste("theta2.", 1:c, sep = "")
+  rownames(output) <- paste("theta1.", 1:c, sep = "")
 
   rm(list = ls()[!(ls() %in% c("output"))])
 
@@ -689,7 +984,7 @@ function(x, pos)
   output <- matrix(data = 0.0, nrow = c, ncol = d, byrow = TRUE)
 
   for (l in 1:c) {
-    output[l, ] <- x@Theta[[pos]][[3 + (l - 1) * 3]]
+    output[l, ] <- x@Theta[[pos]][[3 + (l - 1) * 4]]
   }
 
   colnames(output) <- NULL
@@ -700,6 +995,36 @@ function(x, pos)
   output
 }) ## a.theta2.all
 
+setMethod("a.theta3.all",
+          signature(x = "REBMIX"),
+function(x, pos)
+{
+  if (!is.wholenumber(pos)) {
+    stop(sQuote("pos"), " integer is requested!", call. = FALSE)
+  }
+
+  length(pos) <- 1
+
+  if ((pos < 1) || (pos > nrow(x@summary))) {
+    stop(sQuote("pos"), " must be greater than 0 and less or equal than ", nrow(x@summary), "!", call. = FALSE)
+  }
+
+  c <- length(x@w[[pos]]); d <- length(x@pdf)
+
+  output <- matrix(data = 0.0, nrow = c, ncol = d, byrow = TRUE)
+
+  for (l in 1:c) {
+    output[l, ] <- x@Theta[[pos]][[4 + (l - 1) * 4]]
+  }
+
+  colnames(output) <- NULL
+  rownames(output) <- paste("theta3.", 1:c, sep = "")
+
+  rm(list = ls()[!(ls() %in% c("output"))])
+
+  output
+}) ## a.theta3.all
+
 setMethod("a.theta2.all",
           signature(x = "REBMVNORM"),
 function(x, pos)
@@ -783,7 +1108,7 @@ function(x, pos)
 
 setMethod("a.summary",
           signature(x = "REBMIX"),
-function(x, pos, col.name)
+function(x, col.name, pos)
 {
   if (!is.wholenumber(pos)) {
     stop(sQuote("pos"), " integer is requested!", call. = FALSE)
@@ -818,10 +1143,9 @@ function(x, pos, col.name)
 }) ## a.summary
 
 ### Panic Branislav.
-
 setMethod("a.summary.EM",
           signature(x = "REBMIX"),
-function(x, pos, col.name)
+function(x, col.name, pos)
 {
   if (!is.wholenumber(pos)) {
     stop(sQuote("pos"), " integer is requested!", call. = FALSE)
@@ -854,7 +1178,6 @@ function(x, pos, col.name)
 
   output
 }) ## a.summary.EM
-
 ### End
 
 setMethod("a.pos", signature(x = "REBMIX"), function(x) x@pos)
@@ -1109,6 +1432,7 @@ setMethod("a.from", signature(x = "RCLRMIX"), function(x) x@from)
 setMethod("a.to", signature(x = "RCLRMIX"), function(x) x@to)
 setMethod("a.EN", signature(x = "RCLRMIX"), function(x) x@EN)
 setMethod("a.ED", signature(x = "RCLRMIX"), function(x) x@ED)
+setMethod("a.Rule", signature(x = "RCLRMIX"), function(x) x@Rule)
 
 setMethod("a.s", signature(x = "RCLS.chunk"), function(x) x@s)
 setMethod("a.levels", signature(x = "RCLS.chunk"), function(x) x@levels)
