@@ -406,15 +406,20 @@ FLOAT WeibullInv(FLOAT Fy, FLOAT Theta, FLOAT Beta)
     return (y);
 } // WeibullInv
 
-// Returns the inverse of the Gumbel c.d.f. for the specified Theta and Beta.
+// Returns the inverse of the Gumbel c.d.f. for the specified Mean, Sigma and Xi.
 
-FLOAT GumbelInv(FLOAT Fy, FLOAT Mean, FLOAT Beta)
+FLOAT GumbelInv(FLOAT Fy, FLOAT Mean, FLOAT Sigma, FLOAT Xi)
 {
-	FLOAT y;
+    FLOAT y;
 
-	y = Mean - Beta * (FLOAT)log((FLOAT)log((FLOAT)1.0 / Fy));
+    if (Xi > Eps) {
+        y = Mean + Sigma * (FLOAT)log((FLOAT)log((FLOAT)1.0 / ((FLOAT)1.0 - Fy)));
+    }
+    else {
+        y = Mean - Sigma * (FLOAT)log((FLOAT)log((FLOAT)1.0 / Fy));
+    }
 
-	return (y);
+    return (y);
 } // GumbelInv
 
 // Returns the error function erf(y). See http://www.nr.com/.
