@@ -46,6 +46,8 @@ typedef struct roughparametertype {
     FLOAT h;     // Mode class width;
     FLOAT ym;    // Mode position.
     FLOAT ymean; // Mean position.
+    FLOAT ymin;  // Minimum position.
+    FLOAT ymax;  // Maximum position.
     FLOAT flm;   // Component conditional empirical density.
     FLOAT klm;   // Component conditional total number of observations.
 } RoughParameterType;
@@ -76,7 +78,7 @@ public:
     char                       *save_;         // Path to the save data file.
     PreprocessingType_e        Preprocessing_; // Preprocessing type.
     int                        cmax_;          // Maximum number of components.
-    int                        cmin_;          // Maximum number of components.
+    int                        cmin_;          // Minimum number of components.
     InformationCriterionType_e Criterion_;     // Information criterion type.
     VariablesType_e            *Variables_;    // Types of variables.
     CompnentDistribution       *IniTheta_;     // Initial component parameters.
@@ -87,12 +89,12 @@ public:
     FLOAT                      *ymax_;         // Maximum observations.
     FLOAT                      ar_;            // Acceleration rate.
     PestraintsType_e           Restraints_;    // Restraints type.
-/// Panic Branislav: fields for EM algorithm.
+/// Panic Branislav
     Emmix                      *EM_;           // Object of class Emmix.
     FLOAT                      EM_TOL_;        // Tolerance for EM algorithm.
     FLOAT                      EM_am_;         // Acceleration multiplier for EM algorithm.
     int                        EM_max_iter_;   // Maximum number of iterations of EM algorithm.
-	int                        EM_K_;          // Number of bins for histogram EM algorithm.
+    int                        EM_K_;          // Number of bins for histogram EM algorithm.
     EmStrategyType_e           EM_strategy_;   // EM strategy utilization.
     EmVariantType_e            EM_variant_;    // Type of EM variant algorithm.
     EmAccelerationType_e       EM_accel_;      // Type of acceleration of standard EM algorithm.
@@ -116,7 +118,7 @@ public:
     int                        *all_K_;        // All processed numbers of bins v or all processed numbers of nearest neighbours k.
     FLOAT                      *all_IC_;       // Information criteria for all processed numbers of bins v or all processed numbers of nearest neighbours k.
     AdditionalParameterType    additional_;    // Additional parameters.
-/// Panic Branislav.
+/// Panic Branislav
     int                        n_iter_;        // Number of iterations performed by EM algorithm for optimal mixture selected.
     int                        n_iter_sum_;    // Total number of iterations performed by EM algorithm. 
 /// End
@@ -137,15 +139,15 @@ public:
     virtual int LogComponentDist(int j, FLOAT **Y, CompnentDistribution *CmpTheta, FLOAT *CmpDist, int *Outlier);
     virtual int EnhancedEstimationKNN(FLOAT **Y, FLOAT nl, CompnentDistribution *RigidTheta, CompnentDistribution *LooseTheta);
     virtual int EnhancedEstimationKDE(FLOAT **Y, FLOAT nl, CompnentDistribution *RigidTheta, CompnentDistribution *LooseTheta);
-    virtual int EnhancedEstimationH(int k, FLOAT **Y, FLOAT nl, CompnentDistribution *RigidTheta, CompnentDistribution *LooseTheta);
+    virtual int EnhancedEstimationH(int k, FLOAT **Y, FLOAT nl, FLOAT *h, CompnentDistribution *RigidTheta, CompnentDistribution *LooseTheta);
     virtual int MomentsCalculation(CompnentDistribution *CmpTheta, FLOAT *FirstM, FLOAT *SecondM);
     virtual int BayesClassificationKNN(FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT **FirstM, FLOAT **SecondM);
     virtual int BayesClassificationKDE(FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT **FirstM, FLOAT **SecondM);
     virtual int BayesClassificationH(int k, FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT **FirstM, FLOAT **SecondM);
     virtual int DegreesOffreedom(int c, CompnentDistribution **MixTheta, int *M);
-/// Panic Branislav: method for invoking EM algorithm.
+/// Panic Branislav
     virtual int EMInitialize();
-    virtual int EMRun(int c, FLOAT *W, CompnentDistribution **MixTheta);
+    virtual int EMRun(int *c, FLOAT *W, CompnentDistribution **MixTheta);
 /// End 
     int MixtureDist(int j, FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT *MixDist);
     int MixtureDist(FLOAT logV, int j, FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT *MixDist);
@@ -153,7 +155,65 @@ public:
     int InformationCriterionKDE(FLOAT logV, FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT *IC, FLOAT *logL, int *M, FLOAT *D);
     int InformationCriterionH(FLOAT logV, int k, FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT *IC, FLOAT *logL, int *M, FLOAT *D);
     int CombineComponents(int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT *tau, int *F, int *T, FLOAT *EN, FLOAT *ED);
+/// Panic Branislav
+    int CombineComponentsDemp(int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT *tau, int *F, int *T, FLOAT *EN, FLOAT *ED);
+/// End
     int REBMIX();
+    int Set(char  **Preprocessing,    // Preprocessing type.
+            int   *cmax,              // Maximum number of components.
+            int   *cmin,              // Minimum number of components.
+            char  **Criterion,        // Information criterion type.
+            int   *d,                 // Number of independent random variables.
+            char  **Variables,        // Types of variables.
+            int   *length_pdf,        // Length of pdf.
+            char  **pdf,              // Parametric family types.
+            int   *length_Theta,      // Length of Theta.
+            int   *length_theta,      // Length of Theta[i].
+            FLOAT *Theta,             // Component parameters.
+            int   *length_K,          // Length of K.
+            int   *K,                 // Numbers of bins v or numbers of nearest neighbours k.
+            int   *length_y0,         // Length of y0.
+            FLOAT *y0,                // Origins.
+            int   *length_ymin,       // Length of ymin.
+            FLOAT *ymin,              // Minimum observations.
+            int   *length_ymax,       // Length of ymax.
+            FLOAT *ymax,              // Maximum observations.
+            FLOAT *ar,                // Acceleration rate.
+            char  **Restraints,       // Restraints type.
+            int   *n,                 // Number of observations.
+            FLOAT *Y,                 // Dataset.
+            char  **EMStrategy,       // Strategy for EM algorithm.
+            char  **EMVariant,        // EM algorithm variant.
+            char  **EMAcceleration,   // Acceleration for the standard EM algorithm.
+            FLOAT *EMTolerance,       // Tolerance for EM algortihm.
+            FLOAT *EMAccelerationMul, // Acceleration rate for Em algorithm.
+            int   *EMMaxIter,         // Maximum number of iterations in EM algorithm.
+            int   *EMK);              // Number of bins for histogram EM algorithm.
+/// Panic Branislav
+    int Get(int   *n_iter,         // Number of iterations for optimal case.
+            int   *n_iter_sum,     // Number of iterations in whole run.
+/// End
+            int   *summary_k,      // Optimal v or optimal k.
+            FLOAT *summary_h,      // Optimal class widths of length d.
+            FLOAT *summary_y0,     // Optimal origins of length d.
+            FLOAT *summary_ymin,   // Optimal minimum observations of length d.
+            FLOAT *summary_ymax,   // Optimal maximum observations of length d.
+            FLOAT *summary_IC,     // Optimal information criterion.
+            FLOAT *summary_logL,   // Log-likelihood.
+            int   *summary_M,      // Degrees of freedom.
+            int   *summary_c,      // Optimal number of components.
+            FLOAT *W,              // Component weights.
+            FLOAT *theta1,         // Component parameters.
+            FLOAT *theta2,         // Component parameters.
+            FLOAT *theta3,         // Component parameters.
+            int   *opt_length,     // Length of opt_c, opt_IC, opt_logL and opt_D.
+            int   *opt_c,          // Numbers of components for optimal v or for optimal k.
+            FLOAT *opt_IC,         // Information criteria for optimal v or for optimal k.
+            FLOAT *opt_logL,       // Log-likelihoods for optimal v or for optimal k.
+            FLOAT *opt_D,          // Totals of positive relative deviations for optimal v or for optimal k.
+            int   *all_length,     // Length of all_K and all_IC.
+            int   *all_K,          // All processed numbers of bins v or all processed numbers of nearest neighbours k.
+            FLOAT *all_IC);        // Information criteria for all processed numbers of bins v or all processed numbers of nearest neighbours k.
     #if (_MAINTAIN_SWITCH)
     int RunTemplateFile(char *file);
     #endif
